#!/usr/bin/env python
# config-changed occurs everytime a new configuration
# value is updated (juju set)

import subprocess
import socket
import yaml
import common
from charmhelpers.core.hookenv import (
    config,
)


def config_changed():
    hostname = socket.gethostname()
    host_conf = common.get_host_specific_config(hostname)
    with open('data.yaml', 'r') as f:
        temp = f.read()
        data = yaml.load(temp, Loader=yaml.loader.BaseLoader)
        if ((config('n1kv-source') != data["n1kv_conf"]["n1kv-source"]) or
            (config('n1kv-version') != data["n1kv_conf"]["n1kv-version"])):
            subprocess.call("hooks/install")
        data["n1kv_conf"]["n1kv-source"] = config('n1kv-source')
        data["n1kv_conf"]["n1kv-version"] = config('n1kv-version')
        if (data["n1kv_conf"]["vsm_conf_from_rel"] == "false"):
            data["n1kv_conf"]["n1kv-vsm-ip"] = host_conf['n1kv-vsm-ip']
            data["n1kv_conf"]["n1kv-vsm-domain-id"] = \
                host_conf['n1kv-vsm-domain-id']
        data["n1kv_conf"]["host_mgmt_intf"] = host_conf['host_mgmt_intf']
        data["n1kv_conf"]["uplink_profile"] = host_conf['uplink_profile']
        data["n1kv_conf"]["node_type"] = host_conf['node_type']
        data["n1kv_conf"]["vtep_in_same_subnet"] = \
            host_conf['vtep_in_same_subnet']
        if 'vtep_config' in host_conf:
            data["n1kv_conf"]["vtep_config"] = host_conf['vtep_config']
    with open('data.yaml', 'w') as f:
        f.write(yaml.dump(data, default_flow_style=True))

    if data["n1kv_conf"]["vtep_in_same_subnet"] == "yes":
        f = open("/etc/sysctl.d/60-n1kv.conf", 'w')
        f.writelines(["net.ipv4.conf.default.rp_filter=2\n",
                      "net.ipv4.conf.all.rp_filter=2\n",
                      "net.ipv4.conf.default.arp_ignore=1\n",
                      "net.ipv4.conf.all.arp_ignore=1\n",
                      "net.ipv4.conf.all.arp_announce=2\n",
                      "net.ipv4.conf.default.arp_announce=2\n"])
        f.close()
        subprocess.call(["sysctl", "-p", "/etc/sysctl.d/60-n1kv.conf"])

    common.update_n1kv_config()
    #Enable all required uplink on the host (ifconfig <interface> up)
    common.enable_uplink(data["n1kv_conf"]["uplink_profile"])
    common.enable_uplink(data["n1kv_conf"]["vtep_config"])


if __name__ == '__main__':
    config_changed()
